from dataclasses import dataclass

from ..apis import OpeniAPI
from ..utils import logger, constants
from ..utils.logger import *

log = logger.setup_logger()


def get_upload_type(cluster: str):
    try:
        cluster = cluster.lower()
        upload_type = constants.FILE.UPLOAD_TYPE.index(cluster)
        return upload_type
    except:
        msg = f"`cluster` 参数为 [gpu,npu] 其中之一，不区分大小写"
        log.error(msg)
        raise ValueError(msg)


def get_cluster(upload_type: str):
    try:
        cluster = constants.FILE.UPLOAD_TYPE[upload_type]
        return cluster
    except:
        msg = f"`upload_type` 参数为 [0,1] 其中之一"
        log.error(msg)
        raise ValueError(msg)


@dataclass
class OpeniDataset:
    repo_id: str
    dataset_id: str = None
    dataset_name: str = None
    dataset_files: list = None

    def __post_init__(self):
        self.api = OpeniAPI()
        if self.dataset_name is None:
            self.get_dataset_current_repo()

    def get_dataset_current_repo(self):
        try:
            response = self.api.dataset_query(repo_id=self.repo_id).json()
            self.dataset_id = response["data"][0]["id"]
            self.dataset_name = response["data"][0]["title"]
        except:
            msg = f"❌ 获取数据集失败，`{self.repo_id}` 仓库尚未创建数据集，请先在网页端创建"
            log.error(msg)
            raise ValueError(msg)

    def get_dataset_file(self, upload_type: int):
        try:
            response = self.api.dataset_query_with_attachment(
                repo_id=self.repo_id,
                upload_type=upload_type,
            ).json()
            d = next(d for d in response["data"] if d["title"] == self.dataset_name)
            self.dataset_files = d["attachments"]
        except:
            msg = f"❌ `{self.repo_id}`-`{self.dataset_name}` 获取数据集文件列表失败，请检查数据集是否存在文件或者集群[gpu,npu]是否填写正确"
            log.error(msg)
            raise ValueError(msg)



import os
from tqdm.auto import tqdm

from ..utils import logger
from ..services import *

log = logger.setup_logger()


def upload_file(
    file: str,
    repo_id: str,
    cluster: str = "npu",
):
    upload_type = get_upload_type(cluster)

    # OpeniFileUpload
    local_file = os.path.expanduser(file)
    if not os.path.exists(local_file):
        msg = f"❌ {local_file} 未找到本地文件"
        log.error(msg)
        raise ValueError(msg)

    openi_repo = OpeniRepo(repo_id=repo_id)
    can_operate = (
        openi_repo.access == "write"
        or openi_repo.is_collaborator
        or openi_repo.is_admin
    )
    if can_operate is False:
        msg = f"`{openi_repo.username}` 无权限操作此仓库数据集 `{openi_repo.full_display_name}`"
        log.error(msg)
        raise ValueError(msg)

    openi_dataset = OpeniDataset(repo_id=openi_repo.repo_id)

    # openifiles
    tiltle_pbar = tqdm(
        # total=os.path.getsize(local_file),
        leave=True,
        # unit="B",
        # unit_scale=True,
        # unit_divisor=1024,
        bar_format="{desc}",
        desc=f"Calculating file md5, this might take a while for large file… ",
        position=0,
    )

    openi_file = OpeniFileUpload(
        local_file=local_file,
        repo_id=openi_repo.repo_id,
        upload_type=upload_type,
        dataset_or_model_id=openi_dataset.dataset_id,
        dataset_or_model_name=openi_dataset.dataset_name,
        target_type="dataset",
        # md5_pbar=tiltle_pbar,
    )
    tiltle_pbar.close()

    # # Display pbar
    display_progress_bar([openi_file])

    # # Upload
    openi_file.upload_with_tqdm()



from dataclasses import dataclass

from ..apis import OpeniAPI
from ..utils import logger

log = logger.setup_logger()


def get_login_user(token: str = None, endpoint: str = None):
    try:
        api = OpeniAPI(token=token, endpoint=endpoint)
        response = api.user_my_info().json()
        # username = response.json()["username"]
        return response, api.endpoint
    except:
        msg = f"❌ 无法获取当前用户信息"
        log.error(msg)
        raise ValueError(msg)


@dataclass
class OpeniRepo:
    repo_id: str

    # api = OpeniAPI()

    def __post_init__(self):
        self.api = OpeniAPI()
        user_info, _ = get_login_user()
        self.current_user = user_info["username"]
        self.is_admin = user_info["is_admin"]
        self.get_repo_by_name()
        self.get_repo_access()
        self.check_repo_collaborator()
        self.is_onwer = self.owner == self.current_user

    def check_repo_collaborator(self):
        try:
            response = self.api.repo_is_colloborator(self.repo_id, self.current_user)
            self.is_collaborator = response == 204
        except:
            msg = (
                f"❌ `{self.repo_id}` 无法获取仓库协作者信息，"
                "请检查仓库是否存在或者repo_id是否正确，"
                "repo_id为仓库链接中的 `拥有者(组织)/仓库名`"
            )
            log.error(msg)
            raise ValueError(msg)

    def get_repo_by_name(self):
        try:
            response = self.api.repo_info(self.repo_id).json()
            self.repo_id = response["full_name"]
            self.repo_uuid = response["id"]
            self.full_display_name = response["full_display_name"]
            self.owner = response["owner"]["username"]
        except:
            msg = (
                f"❌ `{self.repo_id}` 无法获取仓库信息，"
                "请检查仓库是否存在或者repo_id是否正确，"
                "repo_id为仓库链接中的 `拥有者(组织)/仓库名`"
            )
            log.error(msg)
            raise ValueError(msg)

    def get_repo_access(self):
        try:
            response = self.api.repo_access(self.repo_id).json()["right"]
            self.access = response
        except:
            msg = (
                f"❌ `{self.repo_id}` 无法获取仓库权限，"
                "请检查仓库是否存在或者repo_id是否正确，"
                "repo_id为仓库链接中的 `拥有者(组织)/仓库名`"
            )
            log.error(msg)
            raise ValueError(msg)



import os

from ..utils import logger, constants
from ..services import *

log = logger.setup_logger()


def download_file(
    file: str,
    repo_id: str,
    cluster: str = "npu",
    save_path: str = constants.PATH.DATASET_SAVE_PATH,
):
    upload_type = get_upload_type(cluster)

    openi_repo = OpeniRepo(repo_id=repo_id)
    if openi_repo.access not in ["read", "write"]:
        msg = f"`{repo_id}` 无权限操作此仓库"
        log.error(msg)
        raise ValueError(msg)

    openi_dataset = OpeniDataset(repo_id=openi_repo.repo_id)
    openi_dataset.get_dataset_file(upload_type=upload_type)
    targe_file = None
    if openi_dataset.dataset_files is not None:
        targe_file = next(f for f in openi_dataset.dataset_files if f["name"] == file)
    if openi_dataset.dataset_files is None or targe_file is None:
        msg = (
            f"❌ `{openi_repo.repo_id}`-`{openi_dataset.dataset_name}` "
            f"数据集内未找到名为{file}({cluster})的压缩文件"
        )
        log.error(msg)
        raise ValueError(msg)

    local_dir = os.path.expanduser(save_path)
    if not os.path.exists(local_dir):
        os.makedirs(local_dir, exist_ok=True)

    openi_file = OpeniFileDownload(
        filename=targe_file["name"],
        size=targe_file["size"],
        local_dir=local_dir,
        repo_id=openi_repo.repo_id,
        dataset_or_model_id=openi_dataset.dataset_id,
        dataset_or_model_name=openi_dataset.dataset_name,
        target_type="dataset",
        upload_type=upload_type,
        uuid=targe_file["uuid"],
    )

    display_progress_bar([openi_file])
    openi_file.download_with_tqdm()



import requests
import json
import os
from pathlib import Path
from dataclasses import dataclass

from .utils import logger, constants

log = logger.setup_logger()


"""
APIS
"""


@dataclass  # (frozen=True)
class OpeniAPI:
    endpoint: str = None
    token: str = None

    def __post_init__(self):
        if self.endpoint is None and self.token is None:
            try:
                saved_token = json.loads(Path(constants.PATH.TOKEN_PATH).read_text())
                self.token = saved_token["token"]
                self.endpoint = saved_token["endpoint"]
            except:
                msg = "❌ 本机尚未登录OpenI，请参考 https://openi.pcl.ac.cn/docs/index.html#/api/cli/login."
                log.error(msg)
                raise FileNotFoundError(msg)

        self.base_url = self.endpoint + constants.API.VERSION

    """
    basics
    """

    def send_api(self, method, route, params=dict(), *args, **kwargs):
        """
        General API call template to OpenI
        """

        url = self.base_url + route
        # 若不需要带token，则调用此函数时明确传入params=None
        if params is not None:
            params["access_token"] = self.token
        response = requests.request(
            method=method,
            url=url,
            params=params,
        )
        try:
            if not kwargs.get("is_collaborator"):
                response.raise_for_status()
            log.http(
                f"{method} {route} {response.status_code} {response.reason} {response.url}"
            )
            return response
        except requests.exceptions.HTTPError as e:
            msg = f"{method} {route} {e}"
            log.warn(msg)
            raise ConnectionError(msg)

    def file_put_upload(self, url, data, upload_type):
        """
        API call to upload a chunk of the file(bytes data) to OpenI
        """
        headers = {"Content-Type": "text/plain"} if upload_type == 0 else {}
        response = requests.request(
            method="PUT",
            url=url,
            data=data,
            headers=headers,
        )
        try:
            response.raise_for_status()
            log.http(
                f"file_put_upload PUT {response.status_code} {response.reason} {response.url}"
            )
        except requests.exceptions.HTTPError as e:
            log.warn(f"{e}")

        return response

    """
    repo and user apis
    """

    def user_my_info(self):
        method = "GET"
        route = f"/user"
        response = self.send_api(method, route)
        log.response(f" {route} {response.json()}")
        return response

    def user_info(self, username):
        method = "GET"
        route = f"/users/{username}"
        response = self.send_api(method, route)
        log.response(f" {route} {response.json()}")
        return response

    def repo_my_list(self):
        method = "GET"
        route = f"/user/repos"
        response = self.send_api(method, route)
        log.response(f" {route} {response.json()}")
        return response

    def repo_list(self, username):
        method = "GET"
        route = f"/users/{username}/repos"
        response = self.send_api(method, route)
        log.response(f" {route} {response.json()}")
        return response

    def repo_info(self, repo_id):
        method = "GET"
        route = f"/repos/{repo_id}"
        response = self.send_api(method, route)
        log.response(f" {route} {response.json()}")
        return response

    def repo_access(self, repo_id):
        method = "GET"
        route = f"/repos/{repo_id}/right"
        response = self.send_api(method, route)
        log.response(f" {route} {response.json()}")
        return response

    def repo_get_collaborators(self, repo_id):
        method = "GET"
        route = f"/repos/{repo_id}/collaborators"
        response = self.send_api(method, route)
        log.response(f" {route} {response.json()}")
        return response

    def repo_is_colloborator(self, repo_id, username):
        method = "GET"
        route = f"/repos/{repo_id}/collaborators/{username}"
        response = self.send_api(method, route, is_collaborator=True)
        log.response(f" {route} {response.status_code}")
        return response.status_code

    """
    dataset api
    """

    def dataset_query(self, repo_id):
        method = "GET"
        route = f"/datasets/{repo_id}"
        response = self.send_api(method, route)
        log.response(f" {route} {response.json()}")
        return response

    def dataset_query_with_attachment(
        self,
        repo_id: str,
        upload_type: int = 0,
    ):
        method = "GET"
        route = f"/datasets/{repo_id}/current_repo"
        params = {"type": upload_type}
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def dataset_file_get_chunks(
        self,
        dataset_id,
        md5,
        filename,
        upload_type,
    ):
        """
        Get information of a file chunk
        """
        method = "GET"
        route = "/attachments/get_chunks"
        params = {
            "dataset_id": dataset_id,
            "md5": md5,
            "file_name": filename,
            "type": upload_type,
        }
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def dataset_file_get_multipart_url(
        self,
        chunk_number,
        chunk_size,
        dataset_id,
        filename,
        upload_type,
        upload_id,
        uuid,
    ):
        method = "GET"
        route = "/attachments/get_multipart_url"
        params = {
            "dataset_id": dataset_id,
            "file_name": filename,
            "type": upload_type,
            "chunkNumber": chunk_number,
            "size": chunk_size,
            "uploadID": upload_id,
            "uuid": uuid,
        }
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def dataset_file_new_multipart(
        self,
        dataset_id,
        md5,
        filename,
        upload_type,
        size,
        total_chunks_count,
    ):
        method = "GET"
        route = "/attachments/new_multipart"
        params = {
            "dataset_id": dataset_id,
            "md5": md5,
            "file_name": filename,
            "type": upload_type,
            "totalChunkCounts": total_chunks_count,
            "size": size,
        }
        response = self.send_api(method, route, params=params)
        if response.json()["result_code"] == "0":
            return response
        else:
            msg = f"{method} {route} {response.json()['msg']}"
            log.warn(msg)
            raise ConnectionError(msg)

    def dataset_file_complete_multipart(
        self,
        dataset_id,
        filename,
        upload_type,
        size,
        upload_id,
        uuid,
    ):
        method = "POST"
        route = "/attachments/complete_multipart"
        params = {
            "dataset_id": dataset_id,
            "file_name": filename,
            "type": upload_type,
            "size": size,
            "uploadID": upload_id,
            "uuid": uuid,
        }
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def dataset_download_file(
        self,
        uuid: str,
        upload_type: int,
    ):
        method = "GET"
        route = f"/attachments/{uuid}"
        params = {"type": upload_type}
        response = self.send_api(
            method,
            route,
            params=params,
            allow_redirects=True,
            stream=True,
        )
        return response

    """
    model management api
    """

    def model_create(
        self,
        repo_id,
        model_name,
        upload_type=1,
        engine=0,
        is_private=True,
    ):
        method = "POST"
        route = f"/repos/{repo_id}/modelmanage/create_local_model"
        params = dict(
            name=model_name,
            type=upload_type,
            engine=engine,
            isPrivate=is_private,
        )
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def model_query_by_id(self, repo_id, model_id):
        method = "GET"
        route = f"/repos/{repo_id}/modelmanage/query_model_byId"
        params = dict(id=model_id)  # {"id": model_id}
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def model_query_by_name(self, repo_id, model_name):
        method = "GET"
        route = f"/repos/{repo_id}/modelmanage/query_model_byName"
        params = dict(name=model_name)  # {"name": model_name}
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def model_file_query_by_id(self, repo_id, model_id):
        method = "GET"
        route = f"/repos/{repo_id}/modelmanage/query_modelfile_for_predict"
        params = dict(id=model_id)  # {"id": model_id}
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def model_download_all(self, repo_id, model_id):
        method = "GET"
        route = f"/repos/{repo_id}/modelmanage/downloadall"
        params = dict(id=model_id)  # {"id": model_id}
        response = self.send_api(method, route, params=params, stream=True)
        return response

    def model_download_single(
        self,
        repo_id,
        model_id,
        filename,
    ):
        method = "GET"
        route = f"/repos/{repo_id}/modelmanage/downloadsingle/{model_id}"
        params = dict(fileName=filename)
        response = self.send_api(
            method, route, params=params, allow_redirects=True, stream=True
        )
        return response

    def model_file_get_chunks(
        self,
        modeluuid,
        md5,
        filename,
        upload_type,
    ):
        method = "GET"
        route = f"/attachments/model/get_chunks"
        params = dict(
            modeluuid=modeluuid,
            md5=md5,
            file_name=filename,
            type=upload_type,
        )
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def model_file_new_multipart(
        self,
        modeluuid,
        md5,
        filename,
        upload_type,
        total_chunks_count,
        size,
    ):
        method = "GET"
        route = f"/attachments/model/new_multipart"
        params = dict(
            modeluuid=modeluuid,
            md5=md5,
            file_name=filename,
            type=upload_type,
            totalChunkCounts=total_chunks_count,
            size=size,
        )
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    def model_file_get_multipart_url(
        self,
        upload_type,
        chunk_number,
        chunk_size,
        upload_id,
        uuid,
    ):
        method = "GET"
        route = f"/attachments/model/get_multipart_url"
        params = dict(
            type=upload_type,
            chunkNumber=chunk_number,
            size=chunk_size,
            uploadID=upload_id,
            uuid=uuid,
        )
        response = self.send_api(method, route, params=params)
        # log.response(f" {route} {response.json()}")
        return response

    def model_file_complete_multipart(
        self,
        modeluuid,
        upload_type,
        upload_id,
        uuid,
    ):
        method = "POST"
        route = f"/attachments/model/complete_multipart"
        params = dict(
            modeluuid=modeluuid,
            type=upload_type,
            uploadID=upload_id,
            uuid=uuid,
        )
        response = self.send_api(method, route, params=params)
        log.response(f" {route} {response.json()}")
        return response

    """
    AI tasks apis
    """

    def aitask_repo_list(
        self,
        repo_id,
        job_type: str = "DEBUG",
        compute_resource: str = "GPU",
        page: int = 1,
    ):
        """
        test
        """
        method = "GET"
        route = f"/{repo_id}/ai_task/list"
        params = dict(
            job_type=job_type,
            compute_resource=compute_resource,
            page=page,
        )
        response = self.send_api(method, route, params=params)
        return response



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="openi-beta",
    version="2.1.10",
    description="A package for openi pypi",
    package_dir={"": "src"},
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://openi.pcl.ac.cn/OpenIOSSG/openi-pypi",
    author="chenzh05",
    author_email="chenzh.ds@outlook.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
    ],
    entry_points={"console_scripts": ["openi = openi.cli:main"]},
    install_requires=["requests", "tqdm"],
    python_requires=">=3.6",
)



import os
from dataclasses import dataclass

from ..apis import OpeniAPI
from ..utils import logger

log = logger.setup_logger()


@dataclass
class OpeniFileDownload:
    """
    Download file object
    """

    filename: str
    size: int
    local_dir: str
    repo_id: str
    dataset_or_model_id: str
    dataset_or_model_name: str
    target_type: str  # ['dataset','model']
    upload_type: int = 1
    uuid: str = None
    filepath: str = None
    pbar: object = None
    downloaded: bool = False

    def __post_init__(self):
        self.api = OpeniAPI()
        self.filepath = os.path.join(self.local_dir, self.filename)
        self.is_download_complete()

    def is_download_complete(self):
        if os.path.exists(self.filepath):
            if self.size == os.path.getsize(self.filepath):
                self.downloaded = True
                return
        self.downloaded = False

    def download_with_tqdm(self):
        if self.downloaded:
            self.pbar.update(self.size)
            self.pbar.set_description_str(self.pbar.desc.replace("Waiting", "Complete"))
            return

        if os.path.exists(self.filepath):
            os.remove(self.filepath)

        self.pbar.set_description_str(self.pbar.desc.replace("Waiting", "Downloading"))
        if self.target_type == "dataset":
            file_stream = self.api.dataset_download_file(
                uuid=self.uuid,
                upload_type=self.upload_type,
            )
        if self.target_type == "model":
            file_stream = self.api.model_download_single(
                repo_id=self.repo_id,
                model_id=self.dataset_or_model_id,
                filename=self.filename,
            )
        block_size = 1024
        with open(self.filepath, "wb") as f:
            for data in file_stream.iter_content(block_size):
                self.pbar.update(len(data))
                f.write(data)

        self.pbar.set_description_str(self.pbar.desc.replace("Downloading", "Complete"))


def rename_existing_file(filepath: str):
    if "tar.gz" in filepath:
        path = filepath.replace(".tar.gz", "")
        suffix = ".tar.gz"
    else:
        path, suffix = os.path.splitext(filepath)

    counter = 1
    while os.path.exists(filepath):
        filepath = "{}({}){}".format(path, counter, suffix)
        counter += 1
    return filepath
